@@ -1,7 +1,7 @@
 /*
 This file is part of Swapspace.
 
-Copyright (C) 2005, Software Industry Industry Promotion Agency (SIPA), Thailand
+Copyright (C) 2005,2006, Software Industry Industry Promotion Agency (SIPA)
 Written by Jeroen T. Vermeulen <jtv@xs4all.nl>.
 
 Swapspace is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with swapspace; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
 #include "env.h"
 
@@ -66,6 +66,18 @@ static memsize_t min_swapsize = 4*MEGA;
  */
 static memsize_t max_swapsize = 2*TERA;
 
+/// Truncate n to a multiple of memory page size
+static memsize_t trunc_to_page(memsize_t n)
+{
+  return n & ~((memsize_t)getpagesize()-1);
+}
+
+/// Round n upwards to multiple of page size
+static memsize_t ext_to_page(memsize_t n)
+{
+  return trunc_to_page(n) + getpagesize()-1;
+}
+
 
 #ifndef NO_CONFIG
 char *set_swappath(long long dummy)
@@ -74,12 +86,12 @@ char *set_swappath(long long dummy)
 }
 char *set_min_swapsize(long long size)
 {
-  min_swapsize = TRUNC_TO_PAGE(size);
+  min_swapsize = trunc_to_page(size);
   return NULL;
 }
 char *set_max_swapsize(long long size)
 {
-  max_swapsize = TRUNC_TO_PAGE(size);
+  max_swapsize = trunc_to_page(size);
   return NULL;
 }
 
@@ -96,7 +108,7 @@ char *set_paranoid(long long dummy)
 bool swaps_check_config(void)
 {
   CHECK_CONFIG_ERR(min_swapsize > max_swapsize);
-  CHECK_CONFIG_ERR(min_swapsize < 10*PAGE_SIZE);
+  CHECK_CONFIG_ERR(min_swapsize < 10*getpagesize());
 
   if (swappath[0] != '/')
   {
@@ -199,7 +211,7 @@ static bool proc_swaps_read_ok = false;
 /// Print status information to stdout
 void dump_stats(void)
 {
-  logm(LOG_INFO, "clock: %lld", (long long)clock);
+  logm(LOG_INFO, "clock: %lld", (long long)runclock);
 
   dump_state();
   dump_memory();
@@ -318,7 +330,7 @@ static bool enable_swapfile(const char file[])
 static memsize_t write_data(int fd, memsize_t bytes, bool persevere)
 {
   // Round upwards to multiple of page size
-  bytes = EXT_TO_PAGE(bytes);
+  bytes = ext_to_page(bytes);
 
   /* Zero buffer before using it to write data to swapfile.  This doesn't do
    * much for security (if an attacker can get to your swapfiles you don't have
@@ -364,7 +376,7 @@ static memsize_t fill_swapfile(const char file[], int fd, memsize_t size)
       // File too big.  Don't try creating files this large again.
       if (likely(bytes > 0 && max_swapsize > bytes))
       {
-        max_swapsize = TRUNC_TO_PAGE(bytes);
+        max_swapsize = trunc_to_page(bytes);
 #ifndef NO_CONFIG
         if (verbose)
 	  logm(LOG_INFO, "Restricting swapfile size to %lld", (long long)bytes);
@@ -396,9 +408,9 @@ static memsize_t fill_swapfile(const char file[], int fd, memsize_t size)
 static memsize_t make_swapfile(const char file[], memsize_t size)
 {
   assert(min_swapsize <= max_swapsize);
-  assert(max_swapsize == TRUNC_TO_PAGE(max_swapsize));
-  assert(min_swapsize == TRUNC_TO_PAGE(min_swapsize));
-  assert(size == TRUNC_TO_PAGE(size));
+  assert(max_swapsize == trunc_to_page(max_swapsize));
+  assert(min_swapsize == trunc_to_page(min_swapsize));
+  assert(size == trunc_to_page(size));
 
   if (unlikely(size < min_swapsize)) return 0;
 
@@ -583,7 +595,7 @@ static bool get_swapfile_status(FILE *fp, struct swapfile_info *result)
 #ifndef NO_CONFIG
 	if (!quiet) logm(LOG_NOTICE, "Detected swapfile '%d'", result->seqno);
 #endif
-	swapfiles[result->seqno].created = clock;
+	swapfiles[result->seqno].created = runclock;
       }
 #ifndef NO_CONFIG
       else if (unlikely(swapfiles[result->seqno].size != result->size))
@@ -601,7 +613,7 @@ static bool get_swapfile_status(FILE *fp, struct swapfile_info *result)
 	      result->seqno,
 	      expected,
 	      found);
-	else if (!quiet && unlikely(found+2*PAGE_SIZE < expected))
+	else if (!quiet && unlikely(found+2*getpagesize() < expected))
 	  log_discrep(LOG_INFO,
 	      "smaller than expected",
 	      result->seqno,
@@ -745,9 +757,10 @@ static int find_free(int last)
 bool alloc_swapfile(memsize_t size)
 {
   /* Round request to page size, then add a bit for swapfile overhead.  Clever
-   * readers will notice that this relies on PAGE_SIZE being a power of two.
+   * readers will notice that this relies on getpagesize() returning a power of
+   * two.
    */
-  size = TRUNC_TO_PAGE(size) + 2*PAGE_SIZE;
+  size = trunc_to_page(size) + 2*getpagesize();
   const int newswap = find_free(sequence_number);
   if (unlikely(swapfiles[newswap].size)) return false;	// No free slot, sorry!
 
